@@ -14,6 +14,9 @@
 #  You should have received a copy of the GNU General Public License
 #  along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
+import xarray as xa
+import arviz as az
+
 from pathlib import Path
 from typing import List, Union, Iterable, Sequence, Optional
 
@@ -23,7 +26,7 @@ from numba import njit, prange
 from numpy import (inf, sqrt, ones, zeros_like, concatenate, diff, log, ones_like, all,
                    clip, argsort, any, s_, zeros, arccos, nan, full, pi, sum, repeat, asarray, ndarray, log10,
                    array, atleast_2d, isscalar, atleast_1d, where, isfinite, arange, unique, squeeze, ceil, percentile,
-                   floor, diag, nanstd)
+                   floor, diag, nanstd, seterr, median)
 from numpy.random import uniform, normal, permutation, multivariate_normal
 from scipy.stats import norm
 
@@ -42,6 +45,7 @@ from ..param.parameter import UniformPrior as U, NormalPrior as N, GammaPrior as
 from .. import QuadraticModel
 from .logposteriorfunction import LogPosteriorFunction
 
+seterr(invalid='ignore')
 
 @njit(cache=False)
 def lnlike_normal(o, m, e):
@@ -160,7 +164,7 @@ class BaseLPF(LogPosteriorFunction):
 
         super().__init__(name=name, result_dir=result_dir)
 
-        self.tm: TransitModel = tm or QuadraticModel(klims=(0.01, 0.75), nk=512, nz=512)
+        self.tm: TransitModel = tm or QuadraticModel(interpolate=False)
         self._tref: float = tref
 
         # Passbands
@@ -232,9 +236,17 @@ class BaseLPF(LogPosteriorFunction):
 
         if isinstance(times, ndarray) and times.ndim == 1 and times.dtype == float:
             times = [times]
+        elif isinstance(times, (list, tuple)):
+            pass
+        else:
+            raise ValueError('The times must be given either as an ndarray or a list of ndarrays.')
 
         if isinstance(fluxes, ndarray) and fluxes.ndim == 1 and fluxes.dtype == float:
             fluxes = [fluxes]
+        elif isinstance(fluxes, (list, tuple)):
+            pass
+        else:
+            raise ValueError('The fluxes must be given either as an ndarray or a list of ndarrays.')
 
         if pbids is None:
             if self.pbids is None:
@@ -325,7 +337,7 @@ class BaseLPF(LogPosteriorFunction):
     def _init_p_planet(self):
         """Planet parameter initialisation.
         """
-        pk2 = [PParameter('k2', 'area_ratio', 'A_s', U(0.05**2, 0.2**2), (0, inf))]
+        pk2 = [PParameter('k2', 'area_ratio', 'A_s', U(0.0025, 0.04), (0, inf))]
         self.ps.add_passband_block('k2', 1, 1, pk2)
         self._pid_k2 = repeat(self.ps.blocks[-1].start, self.npb)
         self._start_k2 = self.ps.blocks[-1].start
@@ -554,31 +566,61 @@ class BaseLPF(LogPosteriorFunction):
                         self.errors[m], self.noise_ids[m], self.nsamples[m], self.exptimes[m])
         self._init_parameters()
 
-    def posterior_samples(self, burn: int = 0, thin: int = 1, derived_parameters: bool = True):
-        df = super().posterior_samples(burn=burn, thin=thin)
-        if derived_parameters:
-            for k2c in df.columns[self._sl_k2]:
-                df[k2c.replace('k2', 'k')] = sqrt(df[k2c])
-            df['a'] = as_from_rhop(df.rho.values, df.p.values)
-            df['inc'] = i_from_baew(df.b.values, df.a.values, 0., 0.)
+    def posterior_samples(self, burn: int = 0, thin: int = 1, derived_parameters: bool = True, arviz: bool = False):
+        if not arviz:
+            df = super().posterior_samples(burn=burn, thin=thin)
+            if derived_parameters:
+                for k2c in df.columns[self._sl_k2]:
+                    df[k2c.replace('k2', 'k')] = sqrt(df[k2c])
+                df['a'] = as_from_rhop(df.rho.values, df.p.values)
+                df['inc'] = i_from_baew(df.b.values, df.a.values, 0., 0.)
 
-            average_ks = sqrt(df.iloc[:, self._sl_k2]).mean(1).values
-            df['t14'] = d_from_pkaiews(df.p.values, average_ks, df.a.values, df.inc.values, 0., 0., 1, kind=14)
-            df['t23'] = d_from_pkaiews(df.p.values, average_ks, df.a.values, df.inc.values, 0., 0., 1, kind=23)
-        return df
-
-    def plot_light_curves(self, method='de', ncol: int = 3, width: float = 2., max_samples: int = 1000, figsize=None,
-                          data_alpha=0.5, ylim=None):
-        nrow = int(ceil(self.nlc / ncol))
-        if method == 'mcmc':
-            df = self.posterior_samples(derived_parameters=False)
-            t0, p = df.tc.median(), df.p.median()
-            fmodel = self.flux_model(permutation(df.values)[:max_samples])
-            fmperc = percentile(fmodel, [50, 16, 84, 2.5, 97.5], 0)
+                average_ks = sqrt(df.iloc[:, self._sl_k2]).mean(1).values
+                df['t14'] = d_from_pkaiews(df.p.values, average_ks, df.a.values, df.inc.values, 0., 0., 1, kind=14)
+                df['t23'] = d_from_pkaiews(df.p.values, average_ks, df.a.values, df.inc.values, 0., 0., 1, kind=23)
+            return df
         else:
-            fmodel = squeeze(self.flux_model(self.de.minimum_location))
-            t0, p = self.de.minimum_location[0], self.de.minimum_location[1]
+            dd = az.from_emcee(self.sampler, var_names=self.ps.names)
+            ds = xa.Dataset()
+            pst = dd.posterior
+            ds['k'] = sqrt(pst.k2)
+            ds['a'] = xa.DataArray(as_from_rhop(pst.rho.values, pst.p.values), coords=pst.k2.coords)
+            ds['inc'] = xa.DataArray(i_from_baew(pst.b.values, ds.a.values, 0., 0.), coords=pst.k2.coords)
+            ds['t14'] = xa.DataArray(d_from_pkaiews(pst.p.values, ds.k.values, ds.a.values, ds.inc.values, 0., 0., 1, kind=14),
+                                     coords=pst.k2.coords)
+            ds['t23'] = xa.DataArray(d_from_pkaiews(pst.p.values, ds.k.values, ds.a.values, ds.inc.values, 0., 0., 1, kind=23),
+                                     coords=pst.k2.coords)
+            dd.add_groups({'derived_parameters': ds})
+            return dd
+
+    def plot_light_curves(self, method='de', ncol: int = 3, width: Optional[float] = None, planet: int = 1,
+                          max_samples: int = 1000, figsize=None, data_alpha=0.5, ylim=None):
+
+        solutions = 'best fit de posterior mc mcmc'.split()
+        if method not in solutions:
+            raise ValueError(f'The "method" needs to be one of {solutions}')
+
+        if width is None:
+            if self.nlc == 1:
+                width = 24 * self.timea.ptp()
+            else:
+                width = 2.0
+
+        ncol = min(ncol, self.nlc)
+        nrow = int(ceil(self.nlc / ncol))
+        tid, pid = self.ps.find_pid(f"tc_{planet}"), self.ps.find_pid(f"p_{planet}")
+        if method in ('mcmc', 'mc', 'posterior'):
+            pvp = self.posterior_samples().posterior.to_array().values.copy().T.reshape([-1, len(self.ps)])
+            t0, p = median(pvp[:, tid]), median(pvp[:, pid])
+            fmodel = self.flux_model(permutation(pvp)[:max_samples])
+            fmperc = percentile(fmodel, [50, 16, 84, 2.5, 97.5], 0)
+        elif method in ('de', 'fit', 'best'):
+            pv = self.de.minimum_location
+            fmodel = squeeze(self.flux_model(pv))
+            t0, p = pv[tid], pv[pid]
             fmperc = None
+        else:
+            raise ValueError
 
         fig, axs = subplots(nrow, ncol, figsize=figsize, constrained_layout=True, sharey='all', sharex='all',
                             squeeze=False)
@@ -590,7 +632,7 @@ class BaseLPF(LogPosteriorFunction):
 
             ax.plot(time, self.fluxes[i], '.', alpha=data_alpha)
 
-            if method == 'de':
+            if method in ('de', 'fit', 'best'):
                 ax.plot(time, fmodel[self.lcslices[i]], 'w', lw=4)
                 ax.plot(time, fmodel[self.lcslices[i]], 'k', lw=1)
             else:
